#!/usr/bin/python3
# -- coding: UTF-8 --

import json
import psutil
import time
import socket
import ctypes
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
gi.require_version('Gtk', '3.0')


class ListControl():
    """
    <node>
        <interface name='com.kylin.kysdk.ListControl'>
             <method name='SetWhiteList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetWhiteList' >
                <arg type='as' direction='out' name='white-list'/>
            </method>
            <method name='ClearWhiteList'>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelWhiteList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
             <method name='SetBlackList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetBlackList' >
                <arg type='as' direction='out' name='black-list'/>
            </method>
            <method name='ClearBlackList'>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelBlackList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")
        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")

    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        time = self.get_host_time
        name = self.get_host_name
        ip = self.get_host_ip
        log_message = "'time':'{}','hostname':'{}','ip':'{}','name':'list-control','lv':'info','message':'{}'".format(
            time, name, ip, message)
        
        try:
            self.log.log_transmit('{' + log_message + '}')
        except Exception:
            print("log message error")


    def SetWhiteList(self, pid: str, vid: str) -> int:
        if len(pid) > 4 or len(vid) > 4:
            return -1
        white_list = self.GetWhiteList()
        pidvid = pid + vid
        if pidvid in white_list:
            return -1
        if pidvid not in white_list:
            file = open("/etc/mdm/white-list", "a")
            file.write(pidvid + "\n")
            file.close()
        try:
            self.log_transmit("添加白名单设备pid: {}, vid: {}".format(pid, vid))
        except Exception:
            print("log message error")

        ret = self.so.add_white_list(
            ctypes.c_char_p(bytes(pid, encoding="utf8")),
            ctypes.c_char_p(bytes(vid, encoding="utf8")))
        return ret

    def GetWhiteList(self) -> list:
        white_list = []
        try:
            file = open("/etc/mdm/white-list", "r")
            white_list_n = file.readlines()
            for white in white_list_n:
                white_list.append(white.strip("\n"))
        except Exception:
            pass
        return white_list

    def ClearWhiteList(self) -> int:
        try:
            self.log_transmit("清空白名单")
        except Exception:
            print("log message error")

        white_list = []
        try:
            file = open("/etc/mdm/white-list", "r")
            white_list_n = file.readlines()
            for white in white_list_n:
                white_list.append(white.strip("\n"))
        except Exception:
            pass
        file = open("/etc/mdm/white-list", "w")
        file.close()
        for pidvid in white_list:
            self.so.enabledevices(
                ctypes.c_char_p(bytes(pidvid[0:4], encoding="utf8")),
                ctypes.c_char_p(bytes(pidvid[4:8], encoding="utf8")))
        return 0

    def DelWhiteList(self, pid: str, vid: str) -> int:
        pidvid = pid + vid + "\n"
        try:
            file = open("/etc/mdm/white-list", "r")
            white_list_n = file.readlines()
            file.close()
        except Exception:
            white_list_n = []

        if pidvid not in white_list_n:
            return - 1
        white_list_n.remove(pidvid)
        try:
            self.log_transmit("删除白名单设备pid: {}, vid: {}".format(pid, vid))
        except Exception:
            print("log message error")

        file = open("/etc/mdm/white-list", "w")
        self.so.enabledevices(
            ctypes.c_char_p(bytes(pid, encoding="utf8")),
            ctypes.c_char_p(bytes(vid, encoding="utf8")))
        for i in white_list_n:
            file.write(i)
        file.close()
        return 0

    def SetBlackList(self, pid: str, vid: str) -> int:
        if len(pid) > 4 or len(vid) > 4:
            return -1
        black_list = self.GetBlackList()
        pidvid = pid + vid
        if pidvid in black_list:
            return -1
        if pidvid not in black_list:
            file = open("/etc/mdm/black-list", "a")
            file.write(pidvid + "\n")
            file.close()

        try:
            self.log_transmit("添加黑名单设备pid: {}, vid: {}".format(pid, vid))
        except Exception:
            print("log message error")

        ret = self.so.add_black_list(
            ctypes.c_char_p(bytes(pid, encoding="utf8")),
            ctypes.c_char_p(bytes(vid, encoding="utf8")))
        return ret

    def GetBlackList(self) -> list:
        black_list = []
        try:
            file = open("/etc/mdm/black-list", "r")
            black_list_n = file.readlines()
            for black in black_list_n:
                black_list.append(black.strip("\n"))
        except Exception:
            pass
        return black_list

    def ClearBlackList(self) -> int:
        try:
            self.log_transmit("清空黑名单")
        except Exception:
            print("log message error")

        black_list = []
        try:
            file = open("/etc/mdm/black-list", "r")
            black_list_n = file.readlines()
            for black in black_list_n:
                black_list.append(black.strip("\n"))
        except Exception:
            pass
        file = open("/etc/mdm/black-list", "w")
        file.close()
        for pidvid in black_list:
            self.so.enabledevices(
                ctypes.c_char_p(bytes(pidvid[0:4], encoding="utf8")),
                ctypes.c_char_p(bytes(pidvid[4:8], encoding="utf8")))
        return 0

    def DelBlackList(self, pid: str, vid: str) -> int:
        pidvid = pid + vid + "\n"
        try:
            file = open("/etc/mdm/black-list", "r")
            black_list_n = file.readlines()
            file.close()
        except Exception:
            black_list_n = []

        if pidvid not in black_list_n:
            return - 1
        black_list_n.remove(pidvid)

        try:
            self.log_transmit("删除黑名单设备pid: {}, vid: {}".format(pid, vid))
        except Exception:
            print("log message error")

        file = open("/etc/mdm/black-list", "w")
        self.so.enabledevices(
            ctypes.c_char_p(bytes(pid, encoding="utf8")),
            ctypes.c_char_p(bytes(vid, encoding="utf8")))
        for i in black_list_n:
            file.write(i)
        file.close()
        return 0


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.kysdk.ListControl", ListControl())
    loop.run()
