#!/usr/bin/python3
# -- coding: UTF-8 --

import psutil
import json
import time
import socket
import ctypes
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
gi.require_version('Gtk', '3.0')


class NetcardControl():
    """
    <node>
        <interface name='com.kylin.kysdk.netcardcontrol'>
            <method name='SetNetcardStatus' >
                <arg type='i' direction='in' name='status'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetNetcardStatus'>
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.func_dict = {
            0: "DisableNetcard",
            1: "EnableNetcard"
        }
        self.so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")
        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")

    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        time = self.get_host_time
        name = self.get_host_name
        ip = self.get_host_ip
        log_message = "'time':'{}','hostname':'{}','ip':'{}','name':'netcard-control','lv':'info','message':'{}'".format(
            time, name, ip, message)

        try:
            self.log.log_transmit('{' + log_message + '}')
        except Exception:
            print("log message error")

    def SetNetcardStatus(self, status: int) -> int:
        """
        status
            0   禁用设备
            1   禁止写入
            2   不管控
        """
        if status in self.func_dict.keys():
            # 通过status调用对应函数
            ret = getattr(self, self.func_dict.get(status))()
            return ret
        else:
            return -1

    def DisableNetcard(self) -> int:
        try:
            self.log_transmit("开启网卡管控")
        except Exception:
            print("log message error")

        if self.GetNetcardStatus() == 0:
            return 0
        file = open("/etc/mdm/net", "w")
        file.write("0")
        file.close()
        # 避免系统无法加载网卡
        self.so.enable_net_card()
        time.sleep(1)
        # 断开wifi连接
        # bus = SystemBus()
        # network_manager = bus.get(".NetworkManager")
        # active_connections = network_manager.ActiveConnections
        # for connection in active_connections:
        #     active_connection = bus.get(".NetworkManager", connection)
        #     if "wireless" in active_connection.Type.lower():
        #         settings = active_connection.Connection
        #         active_settings = bus.get(".NetworkManager", settings)
        #         active_settings.Delete()
        self.so.disable_acpi_net_card()
        self.so.disable_usb_net_card()
        return 0

    def EnableNetcard(self) -> int:
        try:
            self.log_transmit("关闭网卡管控")
        except Exception:
            print("log message error")
            
        if self.GetNetcardStatus() == 1:
            return 0

        file = open("/etc/mdm/net", "w")
        file.write("1")
        file.close()
        self.so.enable_net_card()
        try:
            netlist_f = open("/etc/mdm/disable-net", "r")
            netlist = set(netlist_f.readlines())
            netlist_f.close()
        except Exception:
            return 0
        for net in netlist:
            self.so.enabledevices(
                ctypes.c_char_p(bytes(net[0:4], encoding="utf8")),
                ctypes.c_char_p(bytes(net[4:8], encoding="utf8")))
        return 0

    def GetNetcardStatus(self) -> int:
        try:
            file = open("/etc/mdm/net", "r")
            value = file.read(1)
            file.close()
        except Exception:
            return 1
        return int(value)


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.kysdk.netcardcontrol", NetcardControl())
    loop.run()
